﻿using Castle.DynamicProxy;
using DotNetCore.CAP;
using Hotline.Share.Mq;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

namespace Hotline.Settings.SystemLogDomain;
public class LoggingInterceptor : IInterceptor
{
    private readonly ICapPublisher _cap;

    public LoggingInterceptor(ICapPublisher cap)
    {
        _cap = cap;
    }

    public void Intercept(IInvocation invocation)
    {
        // 检查实现类的方法是否有 LogToDatabaseAttribute
        var methodInfo = invocation.MethodInvocationTarget ?? invocation.Method;
        var logAttribute = methodInfo.GetCustomAttributes(typeof(LogToDataAttribute), true)
                                     .FirstOrDefault() as LogToDataAttribute;

        SystemLog entity = null;
        if (logAttribute != null)
        {
            entity = new SystemLog
            {
                Name = invocation.Method.Name,
            };
            // 遍历所有参数
            for (int i = 0;i < invocation.Arguments.Length;i++)
            {
                var argument = invocation.Arguments[i];
                if (argument == null) continue;
                Type argType = argument.GetType();

                // 遍历参数的所有属性及其值
                var properties = argType.GetProperties(BindingFlags.Public | BindingFlags.Instance);
                foreach (var property in properties)
                {
                    var value = property.GetValue(argument);
                    if (value != null && property.Name.ToUpper() == logAttribute.ExternalIdName.ToUpper())
                    {
                        entity.ExternalId = value.ToString();
                        break;
                    }
                }
            }
        }

        // 执行实际方法
        invocation.Proceed();

        if (logAttribute != null && entity != null)
        {
            _cap.Publish(EventNames.LoggingInterceptor, entity);
        }
    }
}
